#include <string.h>
#include <stdio.h>
#include <stdlib.h>


#define INITIAL_CAPACITY	10
#define INCREMENT			2

typedef struct
{
    char *data;
    int capacity, length;
} string;


string *allocateString()
{
    string *result = (string*) malloc(sizeof(string));
    if (result != NULL)
    {
        result->data = NULL;
        result->capacity = 0;
        result->length = 0;
    }
    return result;
}


/*
 * Increases the capacity of this string, if necessary, to ensure that it can
 * hold at least the number of characters specified by the minimum capacity 
 * argument.
 */
void ensureCapacity(string *str, int minCapacity)
{
    if (str->capacity >= minCapacity)
        return;
    str->data = (char*) realloc(str->data, minCapacity);
    if (str->data == NULL)
    {
        printf("Fatal error: memeory allocation failed!\n");
        exit(1);
    }
    str->capacity = minCapacity;
}

/*
 * Converts a null terminated C into a string structure
 * 
 * Parameters:
 *     cstr - the null terminated C string
 *
 * Assumes:
 *     cstr != NULL
 * 
 * Return:
 *     A newly allocated string containing the same data as the C string, or 
 *     NULL if an error occured.
 */
string *convertCString(const char *cstr)
{
    string *result = NULL;
    int cstrlen = 0;

    result = allocateString();
    if (result != NULL)
    {
        cstrlen = strlen(cstr);
        ensureCapacity(result, cstrlen);
        memcpy(result->data, cstr, cstrlen);
        result->length = cstrlen;
    }
    return result;
}

string* append(string *destination, const string *source)
{
    ensureCapacity(destination, destination->length + source->length);
    memcpy(destination->data + destination->length, source->data, source->length);
	destination->length += source->length;
	return destination;
}

void freeString(string *str)
{
	if (str != NULL) 
	{
		if (str->data != NULL)
			free(str->data);
		free(str);
	}		
}


int main()
{
	string * str = NULL;
	
	str = convertCString("I am a dynamic string");
	if (str == NULL)
	{
		freeString(str);
		return 1;
	}
	
	// print(str);
	freeString(str);

	return 0;
}